% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metadata.R
\name{dashboard_terms}
\alias{dashboard_terms}
\title{Search and retrieve statistical term metadata}
\usage{
dashboard_terms(
  query = NULL,
  category = NULL,
  indicator_code = NULL,
  stat_code = NULL,
  modified_from = NULL,
  modified_to = NULL,
  lang = dash_default("lang", "en"),
  timeout = dash_default("timeout", 30),
  retries = dash_default("retries", 3)
)
}
\arguments{
\item{query}{Optional partial match for a term name.}

\item{category, indicator_code, stat_code}{Optional metadata filters.}

\item{modified_from, modified_to}{Optional metadata update dates in
\code{YYYYMMDD} format.}

\item{lang}{Response language, \code{"en"} or \code{"jp"}.}

\item{timeout}{Request timeout in seconds.}

\item{retries}{Number of retries for transient failures.}
}
\value{
A tibble of statistical terms and definitions.
}
\description{
Search and retrieve statistical term metadata
}
\examples{
\dontrun{
dashboard_terms(category = "0201", lang = "en")
dashboard_terms(query = intToUtf8(c(20154, 21475)), lang = "jp")
}
}
