% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metadata.R
\name{dashboard_surveys}
\alias{dashboard_surveys}
\title{Search and retrieve statistical-survey metadata}
\usage{
dashboard_surveys(
  query = NULL,
  indicator_code = NULL,
  stat_code = NULL,
  modified_from = NULL,
  modified_to = NULL,
  lang = dash_default("lang", "en"),
  timeout = dash_default("timeout", 30),
  retries = dash_default("retries", 3)
)
}
\arguments{
\item{query}{Optional partial match for a survey name.}

\item{indicator_code}{Optional vector of up to 50 indicator codes.}

\item{stat_code}{Optional statistical survey code.}

\item{modified_from, modified_to}{Optional metadata update dates in
\code{YYYYMMDD} format.}

\item{lang}{Response language, \code{"en"} or \code{"jp"}.}

\item{timeout}{Request timeout in seconds.}

\item{retries}{Number of retries for transient failures.}
}
\value{
A tibble of statistical surveys.
}
\description{
Search and retrieve statistical-survey metadata
}
\examples{
\dontrun{
dashboard_surveys(query = "Population Census", lang = "en")
}
}
