% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metadata.R
\name{dashboard_events}
\alias{dashboard_events}
\title{Search and retrieve social-event metadata}
\usage{
dashboard_events(
  time = NULL,
  time_from = NULL,
  time_to = NULL,
  level = NULL,
  category = NULL,
  modified_from = NULL,
  modified_to = NULL,
  lang = dash_default("lang", "en"),
  timeout = dash_default("timeout", 30),
  retries = dash_default("retries", 3)
)
}
\arguments{
\item{time, time_from, time_to}{Optional eight-character API time codes, such
as \code{"20240100"}, \code{"20241Q00"}, \code{"2024CY00"}, or \code{"2024FY00"}.}

\item{level}{Event importance: \code{"low"}, \code{"medium"}, or \code{"high"}; API
codes 1--3 are also accepted.}

\item{category}{Optional Statistics Dashboard category code.}

\item{modified_from, modified_to}{Optional metadata update dates in
\code{YYYYMMDD} format.}

\item{lang}{Response language, \code{"en"} or \code{"jp"}.}

\item{timeout}{Request timeout in seconds.}

\item{retries}{Number of retries for transient failures.}
}
\value{
A tibble with one row per event-category association.
}
\description{
Search and retrieve social-event metadata
}
\examples{
\dontrun{
dashboard_events(category = "0201", level = "high")
}
}
